@@ -24,9 +24,8 @@
 #'   in the set of FCS files. By default is \code{NULL} and the name is
 #'   retrieved automatically.
 #' @param timestep Numerical value that specifies the time step in seconds. In
-#'   other words, it tells how many seconds one unit of
-#'   time corresponds to. By default is \code{NULL} and the value is
-#'   retrieved automatically.
+#'   other words, it tells how many seconds one unit of time corresponds to. By
+#'   default is \code{NULL} and the value is retrieved automatically.
 #' @param second_fractionFR The fraction of a second that is used to split the
 #'   time channel in order to recreate the flow rate. Set it to
 #'   \code{"timestep"} if you wish to recreate the flow rate at the maximum
@@ -34,16 +33,17 @@
 #'   the timestep corresponds to 0.01, however, to shorten the running time of
 #'   the analysis the fraction used by default is 0.1, corresponding to 1/10 of
 #'   a second.
-#' @param deviationFR The metrics used to calculate the dispersion of a flow rate
-#'   fluctuation. Choose between MAD and IQR. Default is MAD and it is more sensitive.
 #' @param alphaFR The level of statistical significance used to accept anomalies
-#'   detected by the ESD method. The default value is \code{0.01}.
+#'   detected by the ESD outlier detection method. The default value is \code{0.01}. 
 #'   Decrease the value to make the flow rate check less sensitive.
-#' @param decompFR Logical indicating whether the flow rate should be decomposed
-#'   in the trend and cyclical components. Default is \code{TRUE} and the ESD
-#'   outlier detection will be executed on the trend component penalized by the
-#'   magnitude of the cyclical component. If it is \code{FALSE} the ESD outlier
-#'   detection will be executed on the original flow rate.
+#' @param ModeDevFR If defined, it will remove parts of the flow rate
+#'   that are a number of standard deviation from the mode of the trend.
+#'   To perform this filter, add the number to multiply to the standard deviation. 
+#'   It is suggested a number between 1 and 2. Default is NULL, hence not performed. 
+#' @param decompFR Default is "cffilter" and it will use the Christiano-Fitzgerald 
+#'   method to calculate the trend and cycle components. Any other value will 
+#'   perform Loess regression to predict the trend line. In this case the cycle 
+#'   component will be the distances from the trend line.
 #' @param ChExcludeFS Character vector with the names or name patterns of the
 #'   channels that you want to exclude from the signal acquisition check. The
 #'   default option, \code{c("FSC", "SSC")}, excludes the scatter parameters. If
@@ -90,6 +90,9 @@
 #' @param folder_results Character string used to name the directory that
 #'   contains the results. The default is \code{"resultsQC"}. If you intend to
 #'   return the results in the working directory use \code{FALSE}.
+#' @param ... additional parameters passed to
+#'   \link[flowCore]{read.flowSet} to provide flexibility over how the
+#'   FCS files are read in.
 #' @return A complete quality control is performed on flow cytometry data in FCS
 #'   format. By default the analysis returns:
 #'
@@ -107,7 +110,7 @@
 #'   3. a single TXT file reporting the percentage of events removed in each FCS
 #'   file.
 #'
-#' @author Gianni Monaco, Chen Hao
+#' @authors Gianni Monaco, Chen Hao
 #' @examples
 #'
 #' ## a sample dataset as flowSet object
@@ -126,16 +129,16 @@
 #' @importFrom scales pretty_breaks
 #' @importFrom graphics hist legend lines
 #' @importFrom methods as is new
-#' @importFrom stats convolve frequency is.ts mad median na.omit qt runif ts tsp
+#' @importFrom stats convolve frequency is.ts mad median na.omit qt runif ts tsp sd loess predict
 #' @importFrom utils write.table
 #' @export
 flow_auto_qc <- function(fcsfiles, remove_from = "all", output = 1,
-     timeCh = NULL, timestep = NULL,second_fractionFR = 0.1, deviationFR = "MAD",
-     alphaFR = 0.01, decompFR = TRUE, ChExcludeFS = c("FSC", "SSC"),
+     timeCh = NULL, timestep = NULL,second_fractionFR = 0.1, alphaFR = 0.01, 
+     ModeDevFR = NULL, decompFR = "cffilter", ChExcludeFS = c("FSC", "SSC"),
      outlier_binsFS = FALSE, pen_valueFS = 500, max_cptFS = 3,
      ChExcludeFM = c("FSC", "SSC"), sideFM = "both", neg_valuesFM = 1,
      html_report = "_QC", mini_report = "QCmini", fcs_QC = "_QC", fcs_highQ = FALSE,
-     fcs_lowQ = FALSE, folder_results = "resultsQC") {
+     fcs_lowQ = FALSE, folder_results = "resultsQC", ...) {
 
     ## load the data
   if( is.character(fcsfiles) ){
@@ -145,9 +148,9 @@ flow_auto_qc <- function(fcsfiles, remove_from = "all", output = 1,
       FileType <- "FCS"
     }
     else if(FileType == "LMD"){
-      set <- read.flowSet(files = fcsfiles, dataset = 2, truncate_max_range = FALSE)
+      set <- read.flowSet(files = fcsfiles, dataset = 2, truncate_max_range = FALSE, ...)
     }else{
-      set <- read.flowSet(files = fcsfiles, truncate_max_range = FALSE)
+      set <- read.flowSet(files = fcsfiles, truncate_max_range = FALSE, ...)
     }
     names <- fcsfiles
   }else if(is(fcsfiles, "flowSet")){
@@ -204,7 +207,9 @@ flow_auto_qc <- function(fcsfiles, remove_from = "all", output = 1,
   out <- list()
 
   for (i in 1:length(set)) {
-    filename_ext <- flowCore::identifier(set[[i]])
+    filename_ext <- basename(flowCore::keyword(set[[i]])$FILENAME)
+    if(is.null(filename_ext))
+      filename_ext <- flowCore::identifier(set[[i]])
     filename <- sub("^([^.]*).*", "\\1", filename_ext)
 
     if (html_report != FALSE) {
@@ -259,7 +264,7 @@ flow_auto_qc <- function(fcsfiles, remove_from = "all", output = 1,
     ### Describe here the arguments for the functions of the flow Rate and Flow Signal
     FR_bin_arg <- list( second_fraction = second_fractionFR, timeCh = timeCh,
                   timestep = timestep)
-    FR_QC_arg <- list( alpha = alphaFR, use_decomp = decompFR, deviation = deviationFR)
+    FR_QC_arg <- list( alpha = alphaFR, decomp = decompFR, ModeDeviation = ModeDevFR)
     FS_bin_arg <- list( binSize = FSbinSize, timeCh = timeCh, timestep = timestep, TimeChCheck = TimeChCheck)
     FS_QC_arg <- list( ChannelExclude = ChExcludeFS, pen_valueFS, max_cptFS, outlier_binsFS )
     FM_QC_arg <- list( ChannelExclude = ChExcludeFM, side= sideFM, neg_values = neg_valuesFM)
